
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import static java.util.concurrent.Executors.newFixedThreadPool;




public class ProfilingArray {

// test results on laptop i7 processor (two cores with hyperthreading)    
// Threads 1 with time: 2936
// Threads 2 with time: 1694
// Threads 3 with time: 1339
// Threads 4 with time: 1276
// Threads 5 with time: 1155
// Threads 10 with time: 1107    
    
    public static void main(String[] args) {

        ProfilingArray tP = new ProfilingArray();

        List<String> results = new ArrayList<>();
        
        results.add(tP.test(1));
        results.add(tP.test(2));
        results.add(tP.test(3));
        results.add(tP.test(4));
        results.add(tP.test(5));
        results.add(tP.test(10));
        
        for (String result : results) {
            System.out.println(result);
        }
       
    }

    ExecutorService threadPool;

    public String test(int numberOfThreads) {

        threadPool = newFixedThreadPool(numberOfThreads);
        List<Future> futures = new ArrayList<>();

        long time = System.currentTimeMillis();        

        for (int i = 1; i <= 10; i++) {
            Callable<String> aCallable = new CallableImpl(i);
            Future<String> submit = threadPool.submit(aCallable);
            futures.add(submit);
        }

        try {
            for (Future future : futures) {
                System.out.println(future.get());
            }
        } catch (Exception ex) {
            System.out.println("Not ready ?? ...");
        }

        threadPool.shutdown(); // shutdown the pool.

        return("Threads " + numberOfThreads + " with time: " + (System.currentTimeMillis() - time));

    }

    private class CallableImpl implements Callable<String> {

        String name;
        boolean sortTweak;

        public CallableImpl(int number) {
            name = "Callable " + number;
        }

        @Override
        public String call() {
            for (int i = 0; i < 2; i++) {
                System.out.println(name + " is doing something");
                final int size = 1000000;
                double[] arr = new double[size];
                sortTweak = false;

                generate(arr);
                System.out.println(name + " [" + i + "] Max: " + getMax(arr));
                System.out.println(name + " [" + i + "] Min: " + getMin(arr));
                System.out.println(name + " [" + i + "] Avg: " + getAvg(arr));
                System.out.println(name + " [" + i + "] Sum: " + getSum(arr));
                System.out.println(name + " [" + i + "] Med: " + getMed(arr));                
            }
            return name + " is done !";
        }

        private void generate(double[] arr) {
            Random random = new Random();
            for (int j = 0; j < arr.length; j++) {
                arr[j] = random.nextDouble();
            }
        }

        private void sortTweakIt(double[] arr) {
            if (!sortTweak) {
                System.out.println("Tweaking ... ");
                
                Arrays.sort(arr);
                sortTweak = true;
            }
        }
        
        private double getMax(double[] arr) {
            sortTweakIt(arr);
            return arr[arr.length-1];
        }
        
        private double getMin(double[] arr) {
            sortTweakIt(arr);
            return arr[0];
        }
        
        private double getSum(double[] arr) {
            double sum = 0;
            for (Double d : arr) {
                sum +=d;
            }
            return sum;
        }
        
        private double getAvg(double[] arr) {
            return getSum(arr) / arr.length;
        }

        private double getMed(double[] arr) {
            sortTweakIt(arr);
            return arr[arr.length/2];
        }
        
        
    }
    
}
